@@ -18,6 +18,7 @@ import time
 import logging
 import pprint
 import shutil
+import sys
 
 from tools import building
 from tools import diagnostics
@@ -31,6 +32,9 @@ from tools.shared import DEBUG, asmjs_mangle
 from tools.shared import treat_as_user_export
 from tools.settings import settings
 
+sys.path.append(path_from_root('third_party'))
+import leb128
+
 logger = logging.getLogger('emscripten')
 
 
@@ -49,7 +53,7 @@ def compute_minimal_runtime_initializer_and_exports(post, exports, receiving):
   declares = 'var ' + ',\n '.join(exports_that_are_not_initializers) + ';'
   post = shared.do_replace(post, '<<< WASM_MODULE_EXPORTS_DECLARES >>>', declares)
 
-  # Generate assignments from all wasm exports out to the JS variables above: e.g. a = asm['a']; b = asm['b'];
+  # Generate assignments from all wasm exports out to the JS variables above: e.g. a = wasmExports['a']; b = wasmExports['b'];
   post = shared.do_replace(post, '<<< WASM_MODULE_EXPORTS >>>', receiving)
   return post
 
@@ -109,14 +113,6 @@ def align_memory(addr):
   return (addr + 15) & -16
 
 
-def get_weak_imports(main_wasm):
-  dylink_sec = webassembly.parse_dylink_section(main_wasm)
-  for symbols in dylink_sec.import_info.values():
-    for symbol, flags in symbols.items():
-      if flags & webassembly.SYMBOL_BINDING_MASK == webassembly.SYMBOL_BINDING_WEAK:
-        settings.WEAK_IMPORTS.append(symbol)
-
-
 def update_settings_glue(wasm_file, metadata):
   maybe_disable_filesystem(metadata.imports)
 
@@ -129,7 +125,7 @@ def update_settings_glue(wasm_file, metadata):
     syms = set(syms).difference(metadata.all_exports)
     settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = sorted(syms)
     if settings.MAIN_MODULE:
-      get_weak_imports(wasm_file)
+      settings.WEAK_IMPORTS += webassembly.get_weak_imports(wasm_file)
 
   settings.WASM_EXPORTS = metadata.all_exports
   settings.WASM_GLOBAL_EXPORTS = list(metadata.namedGlobals.keys())
@@ -459,11 +455,6 @@ def finalize_wasm(infile, outfile, memfile, js_syms):
     # wasm2js requires full legalization (and will do extra wasm binary
     # later processing later anyhow)
     modify_wasm = True
-  if settings.GENERATE_SOURCE_MAP:
-    building.emit_wasm_source_map(infile, infile + '.map', outfile)
-    building.save_intermediate(infile + '.map', 'base_wasm.map')
-    args += ['--output-source-map-url=' + settings.SOURCE_MAP_BASE + os.path.basename(outfile) + '.map']
-    modify_wasm = True
   if settings.DEBUG_LEVEL >= 2 or settings.ASYNCIFY_ADD or settings.ASYNCIFY_ADVISE or settings.ASYNCIFY_ONLY or settings.ASYNCIFY_REMOVE or settings.EMIT_SYMBOL_MAP or settings.EMIT_NAME_SECTION:
     args.append('-g')
   if settings.WASM_BIGINT:
@@ -506,13 +497,43 @@ def finalize_wasm(infile, outfile, memfile, js_syms):
   if settings.DEBUG_LEVEL >= 3:
     args.append('--dwarf')
 
-  metadata = get_metadata(infile, outfile, modify_wasm, args)
-  if modify_wasm:
-    building.save_intermediate(infile, 'post_finalize.wasm')
-  elif infile != outfile:
+  if infile != outfile:
     shutil.copy(infile, outfile)
+
   if settings.GENERATE_SOURCE_MAP:
-    building.save_intermediate(infile + '.map', 'post_finalize.map')
+    building.emit_wasm_source_map(infile, outfile + '.map', outfile)
+    building.save_intermediate(outfile + '.map', 'base_wasm.map')
+    base_url = settings.SOURCE_MAP_BASE + os.path.basename(outfile) + '.map'
+    if modify_wasm:
+      # If we are already modifying, just let Binaryen add the sourcemap URL
+      args += ['--output-source-map-url=' + base_url]
+    else:
+      # Otherwise use objcopy. This avoids re-encoding the file (thus
+      # preserving DWARF) and is faster.
+
+      # Create a file with the contents of the sourceMappingURL section
+      with shared.get_temp_files().get_file('.bin') as url_file:
+        utils.write_binary(url_file,
+                           leb128.u.encode(len(base_url)) + base_url.encode('utf-8'))
+        cmd = [shared.LLVM_OBJCOPY,
+               '--add-section',
+               'sourceMappingURL=' + url_file,
+               infile]
+        shared.check_call(cmd)
+
+  if not settings.GENERATE_DWARF or not settings.EMIT_PRODUCERS_SECTION:
+    # For sections we no longer need, strip now to speed subsequent passes
+    building.save_intermediate(outfile, 'strip.wasm')
+    sections = ['producers'] if not settings.EMIT_PRODUCERS_SECTION else []
+    building.strip(infile, outfile, debug=not settings.GENERATE_DWARF,
+                   sections=sections)
+
+  metadata = get_metadata(outfile, outfile, modify_wasm, args)
+  if modify_wasm:
+    building.save_intermediate(outfile, 'post_finalize.wasm')
+
+  if settings.GENERATE_SOURCE_MAP:
+    building.save_intermediate(outfile + '.map', 'post_finalize.map')
 
   if memfile:
     # we have a separate .mem file. binaryen did not strip any trailing zeros,
@@ -708,8 +729,9 @@ def create_sending(metadata, library_symbols):
 
   sorted_items = sorted(send_items_map.items())
   prefix = ''
-  if settings.USE_CLOSURE_COMPILER:
-    # This prevents closure compiler from minifying the field names in this object.
+  if settings.MAYBE_CLOSURE_COMPILER:
+    # This prevents closure compiler from minifying the field names in this
+    # object.
     prefix = '/** @export */\n  '
   return '{\n  ' + ',\n  '.join(f'{prefix}{k}: {v}' for k, v in sorted_items) + '\n}'
 
@@ -755,7 +777,7 @@ def make_export_wrappers(function_exports, delay_assignment):
       args = ', '.join(args)
       wrapper += f"({args}) => ({mangled} = {exported}wasmExports['{name}'])({args});"
     else:
-      wrapper += 'asm["%s"]' % name
+      wrapper += 'wasmExports["%s"]' % name
 
     wrappers.append(wrapper)
   return wrappers
@@ -763,7 +785,7 @@ def make_export_wrappers(function_exports, delay_assignment):
 
 def create_receiving(function_exports):
   # When not declaring asm exports this section is empty and we instead programatically export
-  # symbols on the global object by calling exportAsmFunctions after initialization
+  # symbols on the global object by calling exportWasmSymbols after initialization
   if not settings.DECLARE_ASM_MODULE_EXPORTS:
     return ''
 
@@ -778,8 +800,8 @@ def create_receiving(function_exports):
       # existing in top level JS scope, i.e.
       # var _main;
       # WebAssembly.instantiate(Module['wasm'], imports).then((output) => {
-      #   var asm = output.instance.exports;
-      #   _main = asm["_main"];
+      #   var wasmExports = output.instance.exports;
+      #   _main = wasmExports["_main"];
       generate_dyncall_assignment = settings.DYNCALLS and '$dynCall' in settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE
       exports_that_are_not_initializers = [x for x in function_exports if x != building.WASM_CALL_CTORS]
 
@@ -790,7 +812,7 @@ def create_receiving(function_exports):
         export_assignment = ''
         if settings.MODULARIZE and should_export:
           export_assignment = f"Module['{mangled}'] = "
-        receiving += [f'{export_assignment}{dynCallAssignment}{mangled} = asm["{s}"]']
+        receiving += [f'{export_assignment}{dynCallAssignment}{mangled} = wasmExports["{s}"]']
     else:
       receiving += make_export_wrappers(function_exports, delay_assignment)
   else:
@@ -815,7 +837,7 @@ def create_module(receiving, metadata, library_symbols):
     module.append('Asyncify.instrumentWasmImports(wasmImports);\n')
 
   if not settings.MINIMAL_RUNTIME:
-    module.append("var asm = createWasm();\n")
+    module.append("var wasmExports = createWasm();\n")
 
   module.append(receiving)
   if settings.SUPPORT_LONGJMP == 'emscripten' or not settings.DISABLE_EXCEPTION_CATCHING:
@@ -874,7 +896,7 @@ def create_pointer_conversion_wrappers(metadata):
     '__cxa_decrement_exception_refcount': '_p',
     '_wasmfs_write_file': '_ppp',
     '__dl_seterr': '_pp',
-    '_emscripten_run_in_main_runtime_thread_js': '___p_',
+    '_emscripten_run_on_main_thread_js': '___p_',
     '_emscripten_proxy_execute_task_queue': '_p',
     '_emscripten_thread_exit': '_p',
     '_emscripten_thread_init': '_p_____',
@@ -891,10 +913,17 @@ def create_pointer_conversion_wrappers(metadata):
     '__get_exception_message': '_ppp',
   }
 
+  for function in settings.SIGNATURE_CONVERSIONS:
+    sym, sig = function.split(':')
+    mapping[sym] = sig
+
   wrappers = '''
-function applySignatureConversions(exports) {
+// Argument name here must shadow the `wasmExports` global so
+// that it is recognised by metadce and minify-import-export-names
+// passes.
+function applySignatureConversions(wasmExports) {
   // First, make a copy of the incoming exports object
-  exports = Object.assign({}, exports);
+  wasmExports = Object.assign({}, wasmExports);
 '''
 
   sigs_seen = set()
@@ -915,8 +944,8 @@ function applySignatureConversions(exports) {
 
   for f in wrap_functions:
     sig = mapping[f]
-    wrappers += f"\n  exports['{f}'] = makeWrapper_{sig}(exports['{f}']);"
-  wrappers += '\n  return exports\n}'
+    wrappers += f"\n  wasmExports['{f}'] = makeWrapper_{sig}(wasmExports['{f}']);"
+  wrappers += 'return wasmExports;\n}'
 
   return wrappers
 
