@@ -6,7 +6,7 @@
 
 #include "arrayUtils.js"
 
-mergeInto(LibraryManager.library, {
+addToLibrary({
 #if TEXTDECODER == 2
   $UTF8Decoder: "new TextDecoder('utf8')",
 #elif TEXTDECODER == 1
@@ -222,7 +222,7 @@ mergeInto(LibraryManager.library, {
 #if ASSERTIONS
     assert(typeof maxBytesToWrite == 'number', 'stringToUTF8(str, outPtr, maxBytesToWrite) is missing the third parameter that specifies the length of the output buffer!');
 #endif
-    return stringToUTF8Array(str, {{{ heapAndOffset('HEAPU8', 'outPtr') }}}, maxBytesToWrite);
+    return stringToUTF8Array(str, HEAPU8, outPtr, maxBytesToWrite);
   },
 
   /**
