@@ -19,6 +19,8 @@ extern "C" {
 __attribute__((nothrow))
 int emscripten_asm_const_int(const char* code, const char* arg_sigs, ...);
 __attribute__((nothrow))
+void* emscripten_asm_const_ptr(const char* code, const char* arg_sigs, ...);
+__attribute__((nothrow))
 double emscripten_asm_const_double(const char* code, const char* arg_sigs, ...);
 
 __attribute__((nothrow))
@@ -42,6 +44,7 @@ void emscripten_asm_const_async_on_main_thread(
 #define EM_ASM_ERROR _Pragma("GCC error(\"EM_ASM does not work in -std=c* modes, use -std=gnu* modes instead\")")
 #define EM_ASM(...) EM_ASM_ERROR
 #define EM_ASM_INT(...) EM_ASM_ERROR
+#define EM_ASM_PTR(...) EM_ASM_ERROR
 #define EM_ASM_DOUBLE(...) EM_ASM_ERROR
 #define MAIN_THREAD_EM_ASM(...) EM_ASM_ERROR
 #define MAIN_THREAD_EM_ASM_INT(...) EM_ASM_ERROR
@@ -66,13 +69,27 @@ void emscripten_asm_const_async_on_main_thread(
 
 // We can use the generic selection C11 feature (that clang supports pre-C11
 // as an extension) to emulate function overloading in C.
-// All pointer types should go through the default case.
+// All other types, including *all* pointer types go through the default case
+#ifdef __wasm64__
+#define LONG_CODE 'j'
+#else
+#define LONG_CODE 'i'
+#endif
 #define _EM_ASM_SIG_CHAR(x) _Generic((x), \
     float: 'f', \
     double: 'd', \
-    int: 'i', \
-    unsigned: 'i', \
-    default: 'i')
+    char: 'i', \
+    unsigned char: 'i', \
+    unsigned short: 'i', \
+    unsigned int: 'i', \
+    unsigned long: LONG_CODE, \
+    unsigned long long: 'j', \
+    signed char: 'i', \
+    signed short: 'i', \
+    signed int: 'i', \
+    signed long: LONG_CODE, \
+    signed long long: 'j', \
+    default: 'p')
 
 // This indirection is needed to allow us to concatenate computed results, e.g.
 //   #define BAR(N) _EM_ASM_CONCATENATE(FOO_, N)
@@ -118,7 +135,7 @@ void emscripten_asm_const_async_on_main_thread(
 #else // __cplusplus
 
 // C++ needs to support vararg template parameter packs, e.g. like in
-// tests/core/test_em_asm_parameter_pack.cpp. Because of that, a macro-only
+// test/core/test_em_asm_parameter_pack.cpp. Because of that, a macro-only
 // approach doesn't work (a macro applied to a parameter pack would expand
 // incorrectly). So we can use a template class instead to build a temporary
 // buffer of characters.
@@ -136,11 +153,18 @@ template<> struct __em_asm_sig<short> { static const char value = 'i'; };
 template<> struct __em_asm_sig<unsigned short> { static const char value = 'i'; };
 template<> struct __em_asm_sig<int> { static const char value = 'i'; };
 template<> struct __em_asm_sig<unsigned int> { static const char value = 'i'; };
+#if __wasm64__
+template<> struct __em_asm_sig<long> { static const char value = 'j'; };
+template<> struct __em_asm_sig<unsigned long> { static const char value = 'j'; };
+#else
 template<> struct __em_asm_sig<long> { static const char value = 'i'; };
 template<> struct __em_asm_sig<unsigned long> { static const char value = 'i'; };
+#endif
 template<> struct __em_asm_sig<bool> { static const char value = 'i'; };
 template<> struct __em_asm_sig<wchar_t> { static const char value = 'i'; };
-template<typename T> struct __em_asm_sig<T*> { static const char value = 'i'; };
+template<> struct __em_asm_sig<long long> { static const char value = 'j'; };
+template<> struct __em_asm_sig<unsigned long long> { static const char value = 'j'; };
+template<typename T> struct __em_asm_sig<T*> { static const char value = 'p'; };
 
 // Explicit support for enums, they're passed as int via variadic arguments.
 template<bool> struct __em_asm_if { };
@@ -178,7 +202,7 @@ const char __em_asm_sig_builder<__em_asm_type_tuple<Args...> >::buffer[] = { __e
 #endif // __cplusplus
 
 // Note: If the code block in the EM_ASM() family of functions below contains a comma,
-// then wrap the whole code block inside parentheses (). See tests/core/test_em_asm_2.cpp
+// then wrap the whole code block inside parentheses (). See test/core/test_em_asm_2.cpp
 // for example code snippets.
 
 #define CODE_EXPR(code) (__extension__({           \
@@ -190,14 +214,18 @@ const char __em_asm_sig_builder<__em_asm_type_tuple<Args...> >::buffer[] = { __e
 // Runs the given JavaScript code on the calling thread (synchronously), and returns no value back.
 #define EM_ASM(code, ...) ((void)emscripten_asm_const_int(CODE_EXPR(#code) _EM_ASM_PREP_ARGS(__VA_ARGS__)))
 
-// Runs the given JavaScript code on the calling thread (synchronously), and returns an integer back.
+// Runs the given JavaScript code on the calling thread (synchronously), and returns an i32 back.
 #define EM_ASM_INT(code, ...) emscripten_asm_const_int(CODE_EXPR(#code) _EM_ASM_PREP_ARGS(__VA_ARGS__))
 
+// Runs the given JavaScript code on the calling thread (synchronously), and returns an pointer back.
+// On wasm32 this is the same as emscripten_asm_const_int but on wasm64 it returns an i64.
+#define EM_ASM_PTR(code, ...) emscripten_asm_const_ptr(CODE_EXPR(#code) _EM_ASM_PREP_ARGS(__VA_ARGS__))
+
 // Runs the given JavaScript code on the calling thread (synchronously), and returns a double back.
 #define EM_ASM_DOUBLE(code, ...) emscripten_asm_const_double(CODE_EXPR(#code) _EM_ASM_PREP_ARGS(__VA_ARGS__))
 
 // Runs the given JavaScript code synchronously on the main browser thread, and returns no value back.
-// Call this function for example to access DOM elements in a pthread when building with -sUSE_PTHREADS.
+// Call this function for example to access DOM elements in a pthread when building with -pthread.
 // Avoid calling this function in performance sensitive code, because this will effectively sleep the
 // calling thread until the main browser thread is able to service the proxied function call. If you have
 // multiple MAIN_THREAD_EM_ASM() code blocks to call in succession, it will likely be much faster to
