@@ -10,6 +10,9 @@
 #include "syscall.h"
 #include "atomic.h"
 #ifdef __EMSCRIPTEN__
+#include "em_task_queue.h"
+#include "thread_mailbox.h"
+#include "threading_internal.h"
 #include <emscripten/threading.h>
 #endif
 #include "futex.h"
@@ -18,22 +21,6 @@
 
 #define pthread __pthread
 
-#ifdef __EMSCRIPTEN__
-#define EM_THREAD_NAME_MAX 32
-
-typedef struct thread_profiler_block {
-	// One of THREAD_STATUS_*
-	_Atomic int threadStatus;
-	// Wallclock time denoting when the current thread state was entered in.
-	double currentStatusStartTime;
-	// Accumulated duration times denoting how much time has been spent in each
-	// state, in msecs.
-	double timeSpentInStatus[EM_THREAD_STATUS_NUMFIELDS];
-	// A human-readable name for this thread.
-	char name[EM_THREAD_NAME_MAX];
-} thread_profiler_block;
-#endif
-
 struct pthread {
 	/* Part 1 -- these fields may be external or
 	 * internal (accessed via asm) ABI. Do not change. */
@@ -41,8 +28,7 @@ struct pthread {
 #ifndef TLS_ABOVE_TP
 	uintptr_t *dtv;
 #endif
-	// TODO(sbc): Implement circular list of threads
-	//struct pthread *prev, *next; /* non-ABI */
+	struct pthread *prev, *next; /* non-ABI */
 	uintptr_t sysinfo;
 #ifndef TLS_ABOVE_TP
 #ifdef CANARY_PAD
@@ -91,7 +77,36 @@ struct pthread {
 	// If --threadprofiler is enabled, this pointer is allocated to contain
 	// internal information about the thread state for profiling purposes.
 	thread_profiler_block * _Atomic profilerBlock;
-	int stack_owned;
+	// The TLS base to use the main module TLS data.  Secondary modules
+	// still require dynamic allocation.
+	void* tls_base;
+	// The lowest level of the proxying system. Other threads can enqueue
+	// messages on the mailbox and notify this thread to asynchronously
+	// process them once it returns to its event loop. When this thread is
+	// shut down, the mailbox is closed (see below) to prevent further
+	// messages from being enqueued and all the remaining queued messages
+	// are dequeued and their shutdown handlers are executed. This allows
+	// other threads waiting for their messages to be processed to be
+	// notified that their messages will not be processed after all.
+	em_task_queue* mailbox;
+	// To ensure that no other thread is concurrently enqueueing a message
+	// when this thread shuts down, maintain an atomic refcount. Enqueueing
+	// threads atomically increment the count from a nonzero number to
+	// acquire the mailbox and decrement the count when they finish. When
+	// this thread shuts down it will atomically decrement the count and
+	// wait until it reaches 0, at which point the mailbox is considered
+	// closed and no further messages will be enqueued.
+	_Atomic int mailbox_refcount;
+	// Whether the thread has executed a `waitAsync` on this pthread struct
+	// and can be notified of new mailbox messages via `Atomics.notify`.
+	// Otherwise the notification has to fall back to the postMessage path.
+	_Atomic int waiting_async;
+#endif
+#ifdef EMSCRIPTEN_DYNAMIC_LINKING
+	// When dynamic linking is enabled, threads use this to facilitate the
+	// synchronization of loaded code between threads.
+	// See emscripten_futex_wait.c.
+	_Atomic char sleeping;
 #endif
 };
 
@@ -131,7 +146,7 @@ enum {
 // XXX Emscripten: The spec allows detecting when multiple write locks would deadlock, so use an extra field
 // _rw_wr_owner to record which thread owns the write lock in order to avoid hangs.
 // Points to the pthread that currently has the write lock.
-#define _rw_wr_owner __u.__p[3]
+#define _rw_wr_owner __u.__vi[3]
 #endif
 #define _b_lock __u.__vi[0]
 #define _b_waiters __u.__vi[1]
@@ -239,7 +254,7 @@ extern hidden unsigned __default_guardsize;
 
 #ifdef __EMSCRIPTEN__
 // Keep in sync with DEFAULT_PTHREAD_STACK_SIZE in settings.js
-#define DEFAULT_STACK_SIZE (2*1024*1024)
+#define DEFAULT_STACK_SIZE (64*1024)
 #else
 #define DEFAULT_STACK_SIZE 131072
 #endif
