@@ -13,7 +13,7 @@ import shutil
 
 __scriptdir__ = os.path.dirname(os.path.abspath(__file__))
 __rootdir__ = os.path.dirname(__scriptdir__)
-sys.path.append(__rootdir__)
+sys.path.insert(0, __rootdir__)
 
 from tools.toolchain_profiler import ToolchainProfiler
 from tools.utils import path_from_root
@@ -37,9 +37,7 @@ func_sig_json = re.compile(r'\["defun", ?"([_\w$]+)",')
 import_sig = re.compile(r'(var|const) ([_\w$]+ *=[^;]+);')
 
 
-def split_funcs(js, just_split=False):
-  if just_split:
-    return [('(json)', line) for line in js.splitlines()]
+def split_funcs(js):
   # split properly even if there are no newlines,
   # which is important for deterministic builds (as which functions
   # are in each chunk may differ, so we need to split them up and combine
@@ -100,9 +98,8 @@ class Minifier:
     self.globs = json.loads(metadata)
 
     if self.symbols_file:
-      with open(self.symbols_file, 'w') as f:
-        for key, value in self.globs.items():
-          f.write(value + ':' + key + '\n')
+      mapping = '\n'.join(f'{value}:{key}' for key, value in self.globs.items())
+      utils.write_file(self.symbols_file, mapping + '\n')
       print('wrote symbol map file to', self.symbols_file, file=sys.stderr)
 
     return code.replace('13371337', '0.0')
@@ -127,8 +124,7 @@ def chunkify(funcs, chunk_size):
   # initialize reasonably, the rest of the funcs we need to split out
   curr = []
   total_size = 0
-  for i in range(len(funcs)):
-    func = funcs[i]
+  for func in funcs:
     curr_size = len(func[1])
     if total_size + curr_size < chunk_size:
       curr.append(func)
@@ -143,7 +139,8 @@ def chunkify(funcs, chunk_size):
   return [''.join(func[1] for func in chunk) for chunk in chunks] # remove function names
 
 
-def run_on_js(filename, passes, extra_info=None, just_split=False, just_concat=False):
+@ToolchainProfiler.profile_block('js_optimizer.run_on_js')
+def run_on_js(filename, passes, extra_info=None):
   with ToolchainProfiler.profile_block('js_optimizer.split_markers'):
     if not isinstance(passes, list):
       passes = [passes]
@@ -179,19 +176,17 @@ def run_on_js(filename, passes, extra_info=None, just_split=False, just_concat=F
       pre = js[:start_funcs + len(start_funcs_marker)]
       post = js[end_funcs + len(end_funcs_marker):]
       js = js[start_funcs + len(start_funcs_marker):end_funcs]
-      if 'asm' not in passes:
-        # can have Module[..] and inlining prevention code, push those to post
-        class Finals:
-          buf = []
+      # can have Module[..] and inlining prevention code, push those to post
+      finals = []
 
-        def process(line):
-          if len(line) and (line.startswith(('Module[', 'if (globalScope)')) or line.endswith('["X"]=1;')):
-            Finals.buf.append(line)
-            return False
-          return True
+      def process(line):
+        if line and (line.startswith(('Module[', 'if (globalScope)')) or line.endswith('["X"]=1;')):
+          finals.append(line)
+          return False
+        return True
 
-        js = '\n'.join(filter(process, js.split('\n')))
-        post = '\n'.join(Finals.buf) + '\n' + post
+      js = '\n'.join(line for line in js.split('\n') if process(line))
+      post = '\n'.join(finals) + '\n' + post
       post = end_funcs_marker + post
   else:
     with ToolchainProfiler.profile_block('js_optimizer.minify_globals'):
@@ -215,7 +210,7 @@ EMSCRIPTEN_FUNCS();
           return False
         return True
 
-      passes = list(filter(check_symbol_mapping, passes))
+      passes = [p for p in passes if check_symbol_mapping(p)]
       asm_shell_pre, asm_shell_post = minifier.minify_shell(asm_shell, 'minifyWhitespace' in passes).split('EMSCRIPTEN_FUNCS();')
       asm_shell_post = asm_shell_post.replace('});', '})')
       pre += asm_shell_pre + '\n' + start_funcs_marker
@@ -233,59 +228,41 @@ EMSCRIPTEN_FUNCS();
 
   with ToolchainProfiler.profile_block('js_optimizer.split'):
     total_size = len(js)
-    funcs = split_funcs(js, just_split)
+    funcs = split_funcs(js)
     js = None
 
   with ToolchainProfiler.profile_block('js_optimizer.split_to_chunks'):
     # if we are making source maps, we want our debug numbering to start from the
     # top of the file, so avoid breaking the JS into chunks
-    cores = shared.get_num_cores()
 
-    if not just_split:
-      intended_num_chunks = int(round(cores * NUM_CHUNKS_PER_CORE))
-      chunk_size = min(MAX_CHUNK_SIZE, max(MIN_CHUNK_SIZE, total_size / intended_num_chunks))
-      chunks = chunkify(funcs, chunk_size)
-    else:
-      # keep same chunks as before
-      chunks = [f[1] for f in funcs]
+    intended_num_chunks = round(shared.get_num_cores() * NUM_CHUNKS_PER_CORE)
+    chunk_size = min(MAX_CHUNK_SIZE, max(MIN_CHUNK_SIZE, total_size / intended_num_chunks))
+    chunks = chunkify(funcs, chunk_size)
 
-    chunks = [chunk for chunk in chunks if len(chunk)]
-    if DEBUG and len(chunks):
-      print('chunkification: num funcs:', len(funcs), 'actual num chunks:', len(chunks), 'chunk size range:', max(map(len, chunks)), '-', min(map(len, chunks)), file=sys.stderr)
+    chunks = [chunk for chunk in chunks if chunk]
+    if DEBUG:
+      lengths = [len(c) for c in chunks]
+      if not lengths:
+        lengths = [0]
+      print('chunkification: num funcs:', len(funcs), 'actual num chunks:', len(chunks), 'chunk size range:', max(lengths), '-', min(lengths), file=sys.stderr)
     funcs = None
 
-    if len(chunks):
-      serialized_extra_info = ''
-      if minify_globals:
-        serialized_extra_info += '// EXTRA_INFO:' + json.dumps(minify_info)
-      elif extra_info:
-        serialized_extra_info += '// EXTRA_INFO:' + json.dumps(extra_info)
-      with ToolchainProfiler.profile_block('js_optimizer.write_chunks'):
-        def write_chunk(chunk, i):
-          temp_file = temp_files.get('.jsfunc_%d.js' % i).name
-          with open(temp_file, 'w') as f:
-            f.write(chunk)
-            f.write(serialized_extra_info)
-          return temp_file
-        filenames = [write_chunk(chunks[i], i) for i in range(len(chunks))]
-    else:
-      filenames = []
+    serialized_extra_info = ''
+    if minify_globals:
+      assert not extra_info
+      serialized_extra_info += '// EXTRA_INFO:' + json.dumps(minify_info)
+    elif extra_info:
+      serialized_extra_info += '// EXTRA_INFO:' + json.dumps(extra_info)
+    with ToolchainProfiler.profile_block('js_optimizer.write_chunks'):
+      def write_chunk(chunk, i):
+        temp_file = temp_files.get('.jsfunc_%d.js' % i).name
+        utils.write_file(temp_file, chunk + serialized_extra_info)
+        return temp_file
+      filenames = [write_chunk(chunk, i) for i, chunk in enumerate(chunks)]
 
   with ToolchainProfiler.profile_block('run_optimizer'):
-    if len(filenames):
-      commands = [config.NODE_JS + [ACORN_OPTIMIZER, f] + passes for f in filenames]
-
-      if os.environ.get('EMCC_SAVE_OPT_TEMP') and os.environ.get('EMCC_SAVE_OPT_TEMP') != '0':
-        for filename in filenames:
-          saved = 'save_' + os.path.basename(filename)
-          while os.path.exists(saved):
-            saved = 'input' + str(int(saved.replace('input', '').replace('.txt', '')) + 1) + '.txt'
-          shutil.copyfile(filename, os.path.join(shared.get_emscripten_temp_dir(), saved))
-
-      filenames = shared.run_multiple_processes(commands, route_stdout_to_temp_files_suffix='js_opt.jo.js')
-
-    for filename in filenames:
-      temp_files.note(filename)
+    commands = [config.NODE_JS + [ACORN_OPTIMIZER, f] + passes for f in filenames]
+    filenames = shared.run_multiple_processes(commands, route_stdout_to_temp_files_suffix='js_opt.jo.js')
 
   with ToolchainProfiler.profile_block('split_closure_cleanup'):
     if closure or cleanup:
@@ -334,19 +311,19 @@ EMSCRIPTEN_FUNCS();
       pre = coutput[:start] + '(' + (USELESS_CODE_COMMENT if has_useless_code_comment else '') + 'function(global,env,buffer) {\n' + pre_2[brace:]
       post = post_1 + end_asm + coutput[end + 1:]
 
-  with ToolchainProfiler.profile_block('write_pre'):
-    filename += '.jo.js'
-    temp_files.note(filename)
-    f = open(filename, 'w')
-    f.write(pre)
-    pre = None
+  filename += '.jo.js'
+  temp_files.note(filename)
 
-  with ToolchainProfiler.profile_block('sort_or_concat'):
-    if not just_concat:
+  with open(filename, 'w') as f:
+    with ToolchainProfiler.profile_block('write_pre'):
+      f.write(pre)
+      pre = None
+
+    with ToolchainProfiler.profile_block('sort_or_concat'):
       # sort functions by size, to make diffing easier and to improve aot times
       funcses = []
       for out_file in filenames:
-        funcses.append(split_funcs(utils.read_file(out_file), False))
+        funcses.append(split_funcs(utils.read_file(out_file)))
       funcs = [item for sublist in funcses for item in sublist]
       funcses = None
       if not os.environ.get('EMCC_NO_OPT_SORT'):
@@ -360,27 +337,15 @@ EMSCRIPTEN_FUNCS();
       for func in funcs:
         f.write(func[1])
       funcs = None
-    else:
-      # just concat the outputs
-      for out_file in filenames:
-        f.write(utils.read_file(out_file))
 
-  with ToolchainProfiler.profile_block('write_post'):
-    f.write('\n')
-    f.write(post)
-    f.write('\n')
-    f.close()
+    with ToolchainProfiler.profile_block('write_post'):
+      f.write('\n')
+      f.write(post)
+      f.write('\n')
 
   return filename
 
 
-@ToolchainProfiler.profile_block('js_optimizer.run_on_js')
-def run(filename, passes, extra_info=None):
-  just_split = 'receiveJSON' in passes
-  just_concat = 'emitJSON' in passes
-  return run_on_js(filename, passes, extra_info=extra_info, just_split=just_split, just_concat=just_concat)
-
-
 def main():
   last = sys.argv[-1]
   if '{' in last:
@@ -388,7 +353,7 @@ def main():
     sys.argv = sys.argv[:-1]
   else:
     extra_info = None
-  out = run(sys.argv[1], sys.argv[2:], extra_info=extra_info)
+  out = run_on_js(sys.argv[1], sys.argv[2:], extra_info=extra_info)
   shutil.copyfile(out, sys.argv[1] + '.jsopt.js')
   return 0
 
